#include "lcd.h"

/* Funkcia zapise jeden bajt po SPI zbernici do zariadenia */
void lcd_write(char data) {
    signed char index = 8;
    char c_data;

    DISPLAY_CSB_PORT &= ~(1 << DISPLAY_CSB_PIN); // Chip-Select do log.0
    c_data = data;

    do {
        _delay_us(6);
        if (c_data & 0x80)
            PORTB |= (1 << 3); // MOSI
        else
            PORTB &= ~(1 << 3);

        _delay_us(5);
        PORTB &= ~(1 << 5); // CLK low
        _delay_us(6);
        PORTB |= (1 << 5); // CLK high

        c_data <<= 1;
        index--;
    } while (index > 0);

    _delay_ms(2);
    DISPLAY_CSB_PORT |= (1 << DISPLAY_CSB_PIN); // CS high
}

void lcd_command(char instruction) {
    DISPLAY_RS_PORT &= ~(1 << DISPLAY_RS_PIN);
    _delay_us(1);
    lcd_write(instruction);
}

void lcd_data(char data) {
    DISPLAY_RS_PORT |= (1 << DISPLAY_RS_PIN);
    _delay_us(7);
    lcd_write(data);
}

void lcd_init(void) {
    DDRB |= (1 << PB3) | (1 << PB5); // MOSI + SCK
    DISPLAY_RS_DDR |= (1 << DISPLAY_RS_PIN);
    DISPLAY_CSB_DDR |= (1 << DISPLAY_CSB_PIN);
    DISPLAY_BKLT_DDR |= (1 << DISPLAY_BKLT_PIN);

    PORTB |= (1 << PB5);
    DISPLAY_CSB_PORT |= (1 << DISPLAY_CSB_PIN);
    DISPLAY_RS_PORT &= ~(1 << DISPLAY_RS_PIN);

    _delay_ms(50);

    lcd_command(0x39);
    _delay_us(50);
    lcd_command(0x1D);
    _delay_us(50);
    lcd_command(0x50);
    _delay_us(50);
    lcd_command(0x6C);
    _delay_ms(500);
    lcd_command(0x7C);
    _delay_us(50);
    lcd_command(0x38);
    _delay_us(50);
    lcd_command(0x0F);
    _delay_us(50);
    lcd_command(0x01);
    _delay_ms(400);
    lcd_command(0x06);
    _delay_us(50);
}

void lcd_putc(char znak) {
    lcd_data(znak);
}

void lcd_puts(char *string) {
    while (*string) {
        lcd_data(*string++);
    }
}

void lcd_setCursor(char row, char col) {
    char address;

    switch (row) {
        case 0: address = 0x00 + col; break;
        case 1: address = 0x10 + col; break;
        case 2: address = 0x20 + col; break;
        default: address = 0x00 + col; break;
    }

    lcd_command(0x80 | address);
}

void lcd_clearline(unsigned char riadok) {
    lcd_setCursor(riadok, 0);
    for (unsigned char i = 0; i < 16; i++) {
        lcd_data(' ');
    }
}

void lcd_clear(void) {
    lcd_clearline(0);
    lcd_clearline(1);
    lcd_clearline(2);
}

void lcd_bklt(char OnOff) {
    if (OnOff)
        DISPLAY_BKLT_PORT |= (1 << DISPLAY_BKLT_PIN);
    else
        DISPLAY_BKLT_PORT &= ~(1 << DISPLAY_BKLT_PIN);
}

void def_znak(unsigned char *ZnakArray, unsigned char Address) {
    lcd_command(0x40 | (Address << 3));
    for (unsigned char i = 0; i < 8; i++) {
        lcd_data(*(ZnakArray + i));
    }
    lcd_command(0x80);
}